# The software in this package is distributed under the GNU General
# Public License version 2 (with a special exception described below).
#
# A copy of GNU General Public License (GPL) is included in this distribution,
# in the file COPYING.GPL.
#
# As a special exception, if other files instantiate templates or use macros
# or inline functions from this file, or you compile this file and link it
# with other works to produce a work based on this file, this file
# does not by itself cause the resulting work to be covered
# by the GNU General Public License.
#
# However the source code for this file must still be made available
# in accordance with section (3) of the GNU General Public License.
#
# This exception does not invalidate any other reasons why a work based
# on this file might be covered by the GNU General Public License.
#
# Copyright (c) 2016-2018 Intra2net AG <info@intra2net.com>

"""Basic functions for on-the-fly arnied cnf-var generator"""

from .cnfline import CnfLine
import os
import tempfile


class BuildCnfVar(object):
    """Basic one the fly arnied cnfvar generator"""
    def __init__(self, name, instance=0, data='', line_no=1):
        self._parent = CnfLine(name, instance, data, line_no)
        self._children = {}            # key: lineno, value: CnfLine

    def __getattr__(self, name):
        # Redirect all unknown attributes to "parent" cnfline
        return getattr(self._parent, name)

    def find_child_line_no(self, name):
        """Look for child line number of child NAME"""
        for lineno, cnfline in self._children.items():
            if cnfline.name == name:
                return lineno

        # Not found
        return 0

    def find_free_line_no(self):
        """Find highest currently in use line number"""
        highest_line_no = self.line_no

        for line_no, unused in self._children.items():
            if line_no > highest_line_no:
                highest_line_no = line_no

        return highest_line_no+1

    def find_free_child_instance(self, name):
        """Find next free child instance of type NAME"""
        highest_instance = -1

        for unused, cnfline in self._children.items():
            if cnfline.name == name and cnfline.instance > highest_instance:
                highest_instance = cnfline.instance

        return highest_instance+1

    def update_cnf(self,
                   name,
                   instance,
                   data,
                   different_parent_line_no=0,
                   force_append=False):
        """Update existing cnfline or create new one"""
        if not force_append:
            child_line_no = self.find_child_line_no(name)
        else:
            child_line_no = 0

        if child_line_no == 0:
            child_line_no = self.find_free_line_no()

        if instance == -1:
            instance = self.find_free_child_instance(name)

        parent_line_no = self._parent.line_no
        if different_parent_line_no:
            parent_line_no = different_parent_line_no

        new_line = CnfLine(name,
                           instance,
                           data,
                           child_line_no,
                           parent_line_no)

        self._children[child_line_no] = new_line
        return child_line_no

    def mark_as_own_parent(self, child_line_no):
        """Remove parent <-> child relationship for
           a given cnf line. We use this heavily
           for the *configure_xxx.py* files"""
        self._children[child_line_no].parent_line_no = 0

    def add_cnf(self,
                name,
                instance,
                data,
                different_parent_line_no=0):
        return self.update_cnf(name,
                               instance,
                               data,
                               different_parent_line_no,
                               force_append=True)

    def del_cnf(self, name):
        """Delete cnfline with name"""
        for lineno, cnfline in list(self._children.items()):
            if cnfline.name == name:
                del self._children[lineno]

    def add_defaults(self, defaults=None):
        """Add default values from a simple dictionary"""
        if defaults is None:
            return

        for name, value in defaults.items():
            self.update_cnf(name, 0, value)

    def __str__(self):
        rtn = str(self._parent) + '\n'
        for unused, value in self._children.items():
            rtn = rtn + str(value) + '\n'

        return rtn

    def save(self, filename):
        """Save string representation to disk."""
        with open(filename, 'w') as out:
            out.write(str(self))
